# 基于SVM的斗地主算法实现

## 数据收集与处理

数据来自小组成员自行人工标注的4000条左右的叫地主记录。数据文件见dataset/call.csv。数据格式如下：

![斗地主数据集截图](https://i.loli.net/2020/06/29/HM7aoiJkbNcwmel.png)

一条数据有17个特征，1个结果。17个特征代表初始17张手牌大小。最后的结果1代表叫地主，0代表不叫。



## 特征向量的定义

经过初步的数据可视化分析，我们发现叫地主和不叫地主主要可以通过**大小王拥有个数、2的个数、炸弹数量**来进行区分。因此，我们定义处理后的特征向量**X = (大小王拥有情况，2的个数，炸弹数量)**，其中小王价值计作1，大王价值计作2，大小王拥有情况等于手牌中大小王价值之和。显然，每个特征值越大，叫地主概率越大，因此这是近似线性可分的数据，适合使用SVM进行二分类。

<img src="https://i.loli.net/2020/06/29/KndP23ZsCGQXceH.png" alt="不叫散点图" style="zoom:67%;" />

<img src="https://i.loli.net/2020/06/29/qrdnDfEpsK3Nvym.png" alt="叫地主散点图" style="zoom:67%;" />



# 基于贪心法的斗地主拆牌策略实现

未经处理的情况下，斗地主的手牌组合、出牌组合和场上其它玩家手牌数的组合情况过多，传统的强化学习算法难以枚举如此多的状态，因此需要对斗地主的状态和动作进行简化。

记玩家当前手牌为S， 当前手牌下的一种合法出牌动作为a，用Q(S, a)代表S和a的情况下，a这个动作拆牌的好坏情况，Q(S，a)值越大，拆牌效果越好。Q(S, a)定义如下：
$$
Q(S, a) = \left\{
\begin{aligned}
length(a) + max(length(a|S-a)) + BombCount(S-a) - BombCount(S), && {length(S-a) > 0} \\
MAXQ, && {length(S-a) = 0}
\end{aligned}
\right.
$$
其中$$length(a)$$代表动作a有几张牌；$$S-a$$代表状态S的牌去掉动作a后剩下的牌，即打出a后的下一个状态；$$BombCount(S)$$代表状态S下的炸弹数量；MAXQ表示一个很大的常数。

公式的前半部分$$length(a) + max(length(a|S-a))$$体现了贪心的思想，认为如果本次出牌和本次出牌后下一次能出的牌数量之和越大，那么这一次拆牌越好。例如S = ‘345667’，如果令a=‘66‘，那么下一次出牌最多只能出单，Q(S， a) = 3，而如果令a='34567'，则下一次出单，Q(S, a) = 6。

再考虑S=’44445‘这种情况，则需要对拆炸弹的行为进行惩罚，因此用$$BombCount(S-a) - BombCount(S)$$来衡量破坏了多少炸弹。

考虑S=’66677‘这种情况，显然此时可以一次性出完所有牌，因此计作最大的Q值。

2和大小王由于无法加入连对，故单独计算。在实际处理中，我们还需要将手牌S分成若干连续的状态，例如将’34567 JJQQ‘ 分成’34567‘和’JJQQ‘两部分。因为拆牌的好坏只会影响到和它连续的牌。



跟牌时，还需要加上上一次出牌的规则限制进行筛选。



# 基于Q-Learning的斗地主出牌策略实现

在斗地主中，我们对出牌和跟牌的状态和动作进行区分，出牌和跟牌的动作定义见下表。因此，需要在一个训练过程中对出牌和跟牌两张Q表进行训练。此外，当玩家0打出一个动作后，需要等待另外玩家1和玩家2做出行动，才能计算出玩家0打出这一动作后得到的奖励和状态。

![状态动作转移图](https://i.loli.net/2020/06/29/6rX3VwPt9ifoaMS.png)

Q-Learning更新Q表的公式如下：

$$Q(s, a) \leftarrow Q(s, a)+\alpha\left[r+\gamma \max _{a^{\prime}} Q\left(s^{\prime}, a^{\prime}\right)-Q(s, a)\right]$$

α越大，智能体越注重过去经验，当α=0时，Q表不再更新；α=1时，Q表将完全抛弃之前学到的经验。γ取值越大，智能体越注重长远收益。当γ取0时，智能体只注重眼下的R(S, a)，γ取1时，智能体将注重所有长远的收益。在算法的实现过程中，我们令α=0.5，γ=0.8。初始化时Q(S, A)均为0。

```
出牌状态是一个长度为12的特征向量。特征的含义以及取值范围如下：
（备注：// 表示整除）
f_min <= f_max

状态说明               属性名                 取值范围（均为整数）
------------------------------------------------------------
f1_min(单张)          solo_min              [0, 3]
f1_max(单张)          solo_max              [0, 3]
f2_min(对子)          pair_min              [0, 2]
f2_max(对子)          pair_max              [0, 2]
三的数量(大于2记作2,含飞机) trios               [0, 2]
最大单顺（长为5） // 5   seq_solo_5             [0, 2]
存在其它牌型(不含4带2)   other_seq_count       [0, 1]
炸弹数量（大于2记作2）    bomb_count            [0, 2]
是否有王炸              rocket                [0, 1]
玩家位置                player               [0, 2]
_hand_to_state(上家手牌数) hand_p             [0, 7]
_hand_to_state(下家手牌数)   hand_n            [0, 7]
------------------------------------------------------------

f1_min(x) = switch mean(x[:2]):  [1, 4] -> 0; [5, 8] -> 1; [9,  12] -> 2; [13, 15] -> 3
f1_max(x) = switch max(x):       [1, 4] -> 0; [5, 8] -> 1; [9,  12] -> 2; [13, 15] -> 3
f2_min(x) = switch mean(x[:2]):  [1, 5] -> 0; [6,10] -> 1; [11, 13] -> 2
f2_max(x) = switch max(x):       [1, 5] -> 0; [6,10] -> 1; [11, 13] -> 2
总共有 (4+3+2+1)*(3+2+1)*3^2*2*3*2*3*8*8 = 1244160 种状态
```

```
AI出牌时动作被化简为以下几个：
【0-4】出单，表示出 强行最小、小、中、大、强行最大的单
【5-7】出对，表示出 小、中、大的对
【8-9】出三，表示出最小、较大的三
【10-12】出炸弹，表示出小炸弹或大炸弹或王炸
【13-14】出长度为5的顺子，表示出较小的或较大的顺子
【15】出其它连对顺子飞机
【16】 出4带2
```

```
跟牌状态如下：

状态说明               属性名                                取值范围（均为整数）
---------------------------------------------------------------------------
最佳拆牌的delta_q       delta_q 越大越拆得差(大于5都记作5)         [0, 5]
玩家身份                player                                 [0, 2]
上一个牌是谁打的         last_combo_owner                       [0, 2]
_hand_to_state(上家手牌数)   hand_p                            [0, 7]
_hand_to_state(上家手牌数)     hand_n                         [0, 7]
上一个牌的数量    last_combo_len(大于5都记作5)                   [0, 5]

---------------------------------------------------------------------------
6*3*3*8*8*6 = 20736
AI跟牌时的动作被化简为以下几个：

0：空过
1-4：跟最小、较小、较大、最大
5：可能拆坏牌的情况下跟最大
6：小炸弹
7：大炸弹
8：王炸
```